// Configuração de constantes
byte const PLRA_LED = 2;  // led do jogador A
byte const PLRA_BTN = 8;  // botão do jogador A
byte const PLRB_LED = 4;  // led do jogador B
byte const PLRB_BTN = 9;  // botão do jogador B
byte const BLINKER = 3;  // porta do LED indicador
byte const BELL = 6;  // porta do buzzer
int const MIN_TIME = 3000;  // tempo mínimo de intervalo entre indicador
int const MAX_TIME = 7000;  // tempo máximo de intervalo entre indicador
int const BELL_CHEAT = 600;

// variáveis a usar no jogo
long last_start = 0;
long score_a = 0;
long score_b = 0;
int multiple = 1;
byte touch = 0;
byte block_A = 0;
byte block_B = 0;

// preparação das portas
void setup()
{
  pinMode(PLRA_LED, OUTPUT);
  pinMode(PLRB_LED, OUTPUT);
  pinMode(PLRA_BTN, INPUT);
  pinMode(PLRB_BTN, INPUT);
  pinMode(BLINKER, OUTPUT);
  pinMode(BELL, OUTPUT);
  intro();
  last_start = nextBlink();
}

// loop do jogo
void loop()
{
  long ms = millis();
  
  // jogo gerado; aceitar input dos jogadores
  if( ms >= last_start && touch == 0 )
  {
    digitalWrite(BLINKER, HIGH);
    
    byte read_A = digitalRead(PLRA_BTN);
    byte read_B = digitalRead(PLRB_BTN);
    
    // leitura do jogador A
    if( read_A == 1 && block_A == 0 )
    {
      score_a += (1 * multiple);
      digitalWrite(PLRA_LED, HIGH);
      touch = 1;
    }
    
    // leitura do jogador B
    if( read_B == 1 && block_B == 0)
    {
      score_b += (1 * multiple);
      digitalWrite(PLRB_LED, HIGH);
      touch = 1;
    }
  }
  else if( ms <= last_start )
  {
    // sistema contra batota
    byte read_A = digitalRead(PLRA_BTN);
    byte read_B = digitalRead(PLRB_BTN);
    
    if( read_A == 1 )
    {
      digitalWrite(PLRA_LED, HIGH);
      tone(BELL, BELL_CHEAT);
      delay(700);
      noTone(BELL);
      digitalWrite(PLRA_LED, LOW);
      block_A = 1;
    }
    else
    {
      block_A = 0;
    }
    
    if( read_B == 1 )
    {
      digitalWrite(PLRB_LED, HIGH);
      tone(BELL, BELL_CHEAT);
      delay(700);
      noTone(BELL);
      digitalWrite(PLRB_LED, LOW);
      block_B = 1;
    }
    else
    {
      block_B = 0;
    }
  }
  
  // gerar novo jogo
  if( ms >= last_start && touch == 1 )
  {
    sound();
    delay(1000);
    digitalWrite(PLRA_LED, LOW);
    digitalWrite(PLRB_LED, LOW);
    digitalWrite(BLINKER, LOW);
    last_start = nextBlink();
    touch = 0;
  }
}

// função para controlar a próxima execução do blinker
long nextBlink()
{
  return millis() + random(MIN_TIME, MAX_TIME);
}

// tocar um som com um buzzer
void sound()
{
  tone(BELL, 1600);
  delay(100);
  noTone(BELL);
  tone(BELL, 2000);
  delay(100);
  noTone(BELL);
  tone(BELL, 2400);
  delay(500);
  noTone(BELL);
}

// tocar o mesmo som, de forma inversa
void r_sound()
{
  tone(BELL, 2400);
  delay(100);
  noTone(BELL);
  tone(BELL, 2000);
  delay(100);
  noTone(BELL);
  tone(BELL, 1600);
  delay(500);
  noTone(BELL);
}

// Animação do jogo (luzes + som)
// as luzes piscam da esquerda para a direita e voltam (ver array seq)
// um som é gerado no inicio da animação e no final é gerado novo som
void intro()
{
  sound();
  
  int seq[] = { PLRA_LED, BLINKER, PLRB_LED, BLINKER, PLRA_LED };
  int sz = 5;
  
  for( byte i = 0; i < sz; i++ )
  {
    digitalWrite(seq[i], HIGH);
    delay(200);
    digitalWrite(seq[i], LOW);
  }
  
  r_sound();
}
